/**
 * @copyright nhcarrigan
 * @license Naomi's Public License
 * @author Naomi Carrigan
 */
import nacl from "tweetnacl";
import { applicationData } from "../config/applicationData.js";
import type { Entitlement } from "../interfaces/entitlement.js";
import type { FastifyRequest } from "fastify";

/**
 * Validates that the webhook request is from Discord by checking the signature.
 * @param request - The Fastify request object containing the webhook data.
 * @returns A boolean indicating whether the webhook signature is valid.
 */
export const validateWebhook = (
  // eslint-disable-next-line @typescript-eslint/naming-convention -- Body must be capitalised for Fastify.
  request: FastifyRequest<{ Body: Entitlement }>,
): boolean => {
  const { application_id: applicationId } = request.body;
  const appData = applicationData[applicationId];
  if (appData === undefined) {
    return false;
  }
  const signature = request.headers["x-signature-ed25519"];
  const timestamp = request.headers["x-signature-timestamp"];
  const { rawBody } = request;
  if (
    signature === undefined
    || timestamp === undefined
    || rawBody === undefined
  ) {
    return false;
  }
  return nacl.sign.detached.verify(
    // eslint-disable-next-line @typescript-eslint/restrict-template-expressions -- Being lazy here, tbh.
    Buffer.from(`${timestamp}${rawBody}`),
    Buffer.from(
      Array.isArray(signature)
        ? signature[0] ?? signature.join("")
        : signature,
      "hex",
    ),
    Buffer.from(process.env.DISCORD_PUBLIC_KEY ?? "", "hex"),
  );
};
